<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 17/03/2016
 * Time: 10:34
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\DevOps\Model\Configurationmanager\Entities as ConfigurationEntities;
use Zeedhi\DevOps\Model\Configurationmanager\Repositories as ConfigurationRepositories;

class Package {

    const RELEASE_STATUS_HOMOLOGATED = ConfigurationEntities\Release::HOMOLOGATED_STATUS;
    const RELEASE_IS_ACTIVE_POSITIVE = ConfigurationEntities\Release::POSITIVE_IS_ACTIVE;
    static $RELEASE_STATUS_NOT_HOMOLOGATED = array(
        ConfigurationEntities\Release::OPENED_STATUS,
        ConfigurationEntities\Release::IN_HOMOLOGATION_STATUS,
        ConfigurationEntities\Release::UPDATE_FAIL_STATUS
    );

    /** @var \Factory\Package $packageFactory */
    protected $packageFactory;
    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var ConfigurationRepositories\Product $productRepository */
    protected $productRepository;
    /** @var ConfigurationRepositories\Release $releaseRepository */
    protected $releaseRepository;
    /** @var ConfigurationRepositories\Package $packageRepository */
    protected $packageRepository;
    /** @var ConfigurationRepositories\PackageContents $packageContentsRepository */
    protected $packageContentsRepository;
    /**
     * @var Environment
     */
    private $environment;

    /**
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param \Factory\Package $packageFactory
     */
    public function __construct(EntityManager $entityManager, Environment $environment, \Factory\Package $packageFactory) {
        $this->entityManager  = $entityManager;
        $this->packageFactory = $packageFactory;
        $this->environment = $environment;

        $this->productRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Product');
        $this->releaseRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Release');
        $this->packageRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Package');
        $this->packageContentsRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'PackageContents');
    }

    /**
     * @param $productVersionId
     * @return array
     */
    public function findPackageByProductVersion($productVersionId) {
        /** @var \Zeedhi\DevOps\Model\Configurationmanager\Repositories\Package $packageRepository */
        $packageRepository = $this->entityManager->getRepository(Utilities::CONFIGURATION_MANAGER_ENTITY_DIR . 'Package');
        $organizationId = $this->environment->getOrganizationId();
        return $packageRepository->getPackageByProductVersion($productVersionId,$organizationId);
    }

    /**
     * @param $productId
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function updateReleasePackagesByProduct($productId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->cleanReleasePackages($productId);
            $product = $this->productRepository->find($productId);
            if (!empty($product)) {
                $this->createNewPackages($product);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $productId
     * @return array
     */
    private function cleanReleasePackages($productId)
    {
        $organizationId = $this->environment->getOrganizationId();
        $packageIds = $this->packageRepository->findReleasePackagesByProduct($productId,$organizationId);
        foreach ($packageIds as $package) {
            $packageId = $package['ID'];
            $package = $this->packageRepository->find($packageId);
            $packageContents = $this->packageContentsRepository->findBy(array('package' => $package));
            foreach ($packageContents as $packageContent) {
                $this->entityManager->remove($packageContent);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($package);
            $this->entityManager->flush();
        }
    }

    /**
     * @param $product
     */
    private function createNewPackages($product)
    {
        $packages = $this->packageRepository->findBy(array('product' => $product));
        $releases = $this->releaseRepository->findBy(array('product' => $product,
            'isActive' => self::RELEASE_IS_ACTIVE_POSITIVE, 'status' => self::$RELEASE_STATUS_NOT_HOMOLOGATED));
        foreach ($packages as $package) {
            $packageContents = $this->packageContentsRepository->findBy(array('package' => $package));
            $this->createNewReleasePackages($releases, $package, $packageContents);
        }
    }

    /**
     * @param $packageContents
     * @param $packageToSave
     */
    private function createNewPackageContents($packageContents, $packageToSave)
    {
        /** @var ConfigurationEntities\PackageContents $packageContent */
        foreach ($packageContents as $packageContent) {
            $packageContentToSave = $this->packageFactory->setPackageContentsRecords(null, $packageToSave,
                $packageContent->getName(), $packageContent->getPath(), $packageContent->getFileName());
            $this->entityManager->persist($packageContentToSave);
        }
    }

    /**
     * @param $releases
     * @param $package
     * @param $packageContents
     */
    private function createNewReleasePackages($releases, $package, $packageContents)
    {
        /** @var ConfigurationEntities\Package $package */
        foreach ($releases as $release) {
            /** @var ConfigurationEntities\Release $release */
            if ($release->getStatus() != self::RELEASE_STATUS_HOMOLOGATED) {
                $packageToSave = $this->packageFactory->setPackageRecords(null, $release, $package->getName(),
                    $package->getPath(), $package->getDescription(), $package->getFileName(), $package->getPathToDeploy());
                $this->entityManager->persist($packageToSave);
                $this->entityManager->flush();
                $this->createNewPackageContents($packageContents, $packageToSave);
            }
        }
    }
} 