<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Doctrine\DBAL\Connection;

class ResourceProject extends EntityRepository {

    const STATUS_LOGICAL_DELETED = LogicalDeleted::STATUS_LOGICAL_DELETED;


    public function getResourceProjects($resourceId)    {
        $query =  <<<SQL
            SELECT * FROM (SELECT DISTINCT P.NAME AS NAME, T.PARENT AS PARENT, RP.PROJECT AS ID, RP.ID AS RESOURCE_PROJECT, P.BEGIN_DATE,
                            P.END_DATE
            FROM        PROJECT P
            INNER JOIN  RESOURCE_PROJECT RP ON P.ID = RP.PROJECT
            LEFT JOIN (
                SELECT P.ID AS PARENT
                FROM        PROJECT P
                INNER JOIN  RESOURCE_PROJECT RP ON P.ID = RP.PROJECT
                WHERE   RP.RESOURCE_C = :resourceId
                AND     P.STATUS = 'A'
            ) T                             ON (T.PARENT = P.PARENT)
            WHERE   RP.RESOURCE_C = :resourceId
            AND     P.STATUS = 'A'
SQL;
        $param = array(
            'resourceId' => $resourceId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query.") P
			ORDER BY P.BEGIN_DATE DESC, P.END_DATE DESC", $param);
    }

    public function getProjectPaths($resourceId){
        $query = <<<SQL
            SELECT  PRO.ID||' - '||PRO.NAME AS NAME, PRO.ID, PRO.PARENT, TAS.CONTROL, REP.ID AS RESOURCE_PROJECT
            FROM    PROJECT PRO
            JOIN    PROJECT_TYPE PT ON PRO.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
            JOIN    TASK_STATUS TAS       ON (PRO.PROJECT_STATUS = TAS.ID)
            JOIN    RESOURCE_PROJECT REP  ON (PRO.ID=REP.PROJECT)
            JOIN    REPOSITORY REPO       ON (REPO.PROJECT = PRO.ID)
            WHERE   TAS.CONTROL       NOT IN (:statusNotIn)
            AND     REP.RESOURCE_C = :RESOURCE_ID
            AND     PRO.STATUS = 'A'
            AND     REPO.STATUS <> :LOGICAL_DELETE
            GROUP BY PRO.NAME, PRO.ID, PRO.PARENT, TAS.CONTROL, REP.ID
            ORDER BY PRO.ID
SQL;
        $param = array(
            'RESOURCE_ID' => $resourceId,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
            'statusNotIn' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_PLANNING,
                Entities\TaskStatus::TASK_STATUS_CONTROL_NOT_INITIALIZED
            ),
        );
        $paramTypes = array(
            'statusNotIn' => Connection::PARAM_STR_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param, $paramTypes);
    }

    public function getResourceProjectPaths($resourceId,$projectId){
        $query = <<<SQL
       SELECT D.LOCAL_DIRECTORY AS PATH,D.LINUX_DIRECTORY, D.LOCAL_DIRECTORY, D.ID AS PATH_ID ,R.NAME AS URL, R.URL AS URL_REPOSITORY, R.ID AS REPOSITORY_ID, P.ID AS PROJECT_ID,
        'P'||P.ID AS PR_PROJECT,R.NAME AS REPOSITORY,P.NAME AS PROJECT, P.ID as "parentLocalConection"
        FROM DIRECTORY D
        INNER JOIN REPOSITORY R ON D.REPOSITORY = R.ID
        INNER JOIN PROJECT P ON R.PROJECT = P.ID
        WHERE D.RESOURCE_T = :RESOURCE_ID
        AND  P.STATUS = 'A'
        AND P.ID = :PROJECT_ID
SQL;
        $param = array(
            'RESOURCE_ID' => $resourceId,
            'PROJECT_ID' => $projectId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function getFollowerInMeeting($projectId){
        $typeWork = Entities\ResourceProject::WORK;
        $query = <<<SQL
            SELECT RES.ID,RES.NAME
              FROM RESOURCE_PROJECT REP
              JOIN RESOURCE_T RES ON(RES.ID = REP.RESOURCE_C)
             WHERE REP.PROJECT = :PROJECT_ID
               AND REP.TYPE    = :WORK_TYPE
          ORDER BY RES.NAME
SQL;
        $param = array(
            'PROJECT_ID' => $projectId,
            'WORK_TYPE' => $typeWork,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }
    
    public function getParentProjectPaths($resourceId,$projectId){
        $query = <<<SQL
            SELECT  PRO.ID||' - '||PRO.NAME AS NAME, PRO.ID, PRO.PARENT, TAS.CONTROL, REP.ID AS RESOURCE_PROJECT
            FROM    PROJECT PRO
                    JOIN    PROJECT_TYPE PT ON PRO.PROJECT_TYPE = PT.ID AND PT.EXTERNALTYPE <> :PROJECT_TYPE_TIME_LINE
                    JOIN    TASK_STATUS TAS       ON (PRO.PROJECT_STATUS = TAS.ID)
                    JOIN    RESOURCE_PROJECT REP  ON (PRO.ID=REP.PROJECT)
                    JOIN    REPOSITORY REPO       ON (REPO.PROJECT = PRO.ID)
            WHERE   TAS.CONTROL       NOT IN (:statusNotIn)
                    AND     REP.RESOURCE_C = :RESOURCE_ID
                    AND     PRO.PARENT = :PARENT_ID
                    AND     PRO.STATUS = 'A'
                    AND     REPO.STATUS <> :LOGICAL_DELETE
            GROUP BY PRO.NAME, PRO.ID, PRO.PARENT, TAS.CONTROL, REP.ID
            ORDER BY PRO.ID
SQL;
        $param = array(
            'RESOURCE_ID' => $resourceId->getId(),
            'PARENT_ID' => $projectId,
            'LOGICAL_DELETE' => self::STATUS_LOGICAL_DELETED,
            'PROJECT_TYPE_TIME_LINE' => Entities\ProjectType::TYPE_PROJECT_CONTROL_TIME_LINE,
            'statusNotIn' => array(
                Entities\TaskStatus::TASK_STATUS_CONTROL_CONCLUDED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_CANCELED,
                Entities\TaskStatus::TASK_STATUS_CONTROL_PLANNING,
                Entities\TaskStatus::TASK_STATUS_CONTROL_NOT_INITIALIZED
            ),
        );
        $paramTypes = array(
            'statusNotIn' => Connection::PARAM_STR_ARRAY,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param, $paramTypes);
    }
    
    public function getReposioryFromProject($projectId){
        $query = <<<SQL
            SELECT * 
            FROM REPOSITORY 
            WHERE PROJECT = :PROJECT_ID
SQL;
        $param = array(
            'PROJECT_ID' => $projectId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }
    
    
    public function getDirectoryFromProject($repositoryId, $projectId, $resourceId){
        $query = <<<SQL
            SELECT D.*, R.PROJECT 
            FROM DIRECTORY D
                JOIN REPOSITORY R           ON R.ID = D.REPOSITORY
            WHERE D.REPOSITORY = :REPOSITORY_ID 
                AND R.PROJECT = :PROJECT_ID
                AND R.STATUS = 'A'
                AND D.RESOURCE_T = :RESOURCE_ID
SQL;
        $param = array(
            'REPOSITORY_ID' => $repositoryId,
            'PROJECT_ID' => $projectId,
            'RESOURCE_ID' => $resourceId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
    }

    public function getProjectsPMN($organization, $userId){
        $query = <<<SQL
            SELECT P.ID, P.NAME, P.PARENT
            FROM PROJECT P
            INNER JOIN RESOURCE_PROJECT RP ON RP.PROJECT = P.ID
            WHERE RP.ORGANIZATION = :ORGANIZATION
            AND RP.RESOURCE_C = :USERID
            ORDER BY P.ID
SQL;
        $param = array(
            'ORGANIZATION' => $organization,
            'USERID' => $userId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $param);
	}
}